	FUNCTION DCADRE(F,A,B,RERR,ERROR,IER)


C	LATEST REVISION	- JULY, 1999

C	PURPOSE			- TO INTEGRATE A FUNCTION USING THE METHOD OF
C					  ROMBERG EXTRAPOLATION OF ORDER 2K

C	SUBROUTINES		- THESE SUBROUTINES ARE BASED UPON THOSE FOUND IN
C					  "NUMERICAL RECIPIES IN FORTRAN 77", SECOND EDITION,
C					  CAMBRIDGE UNIVERSITY PRESS, P. 134, 131,AND 103,
C					  RESP.
C		QROMB		  SUBROUTINE TO PERFORM THE INTEGRATION OF FUNCTION
C					  FROM A TO B.  EPS IS THE FRACTIONAL ACCURACY 
C					  DESIRED, JMAX IS LIMIT ON TOTAL NUMBER OF STEPS, 
C					  AND K IS THE NUMBER OF POINTS USED IN EXTRAPOLATION
C					  USES -
C						TRAPZD - SUBROUTINE WHICH COMPUTES THE NTH STAGE
C							     OF REFINEMENT OF EXTENDED TRAPEZOIDAL
C							     RULE.  EVALUATES FUNCTION FROM A TO B
C						POLINT - SUBROUTINE WHICH GENERATES VALUES
C								 FOR CONVERGENCE CRITERIA

C	FUNCTIONS		-
C		FUNC		  USER SUPPLIED FUNCTION SUBRPOGRAMS WHICH
C					  DEFINES FUNCTIONS TO BE INTEGRATED, DECLARED
C					  EXTERNAL IN THE CALLING PROGRAM

C	ERROR CODES		- 100 MEANS THAT RERR IS GREATER THAN 0.1
C					  OR LESS THAN 0.0
C					- 101 MEANS THAT CONVERGENCE CRITERIA HAS
C					  NOT BEEN MET AFTER MAXIMUM NUMBER OF ITERATIONS 
C					  HAVE BEEN PERFORMED
C					- 102 MEANS THAT THERE WAS A FAILURE IN THE 
C					  SUBROUTINE POLINT
 

	IMPLICIT NONE

	INTEGER*4	IER
	
	REAL*8		F,A,B,RERR,ERROR,DCADRE,s
	
	EXTERNAL	F

		call qromb(F,A,B,s,RERR)

		DCADRE = s
	
	END
	  	

      SUBROUTINE qromb(func,a,b,ss,RERR)

	IMPLICIT NONE

      INTEGER*4	JMAX,JMAXP,K,KM

      REAL*8		a,b,func,ss,RERR

      EXTERNAL	func

      PARAMETER	(JMAX=20, JMAXP=JMAX+1, K=5, KM=K-1)

      INTEGER*4	j, IER

      REAL*8		dss,h(JMAXP),s(JMAXP)

		IER = 0
		IF (RERR .GT. 0.1d0 .OR. RERR .LT. 0.0d0) GOTO 2000
		h(1)=1.d0
		DO 11 j=1,JMAX
			call trapzd(func,a,b,s(j),j)
			IF (j.ge.K) THEN
				call polint(h(j-KM),s(j-KM),K,0.d0,ss,dss)
				IF (DABS(dss).le.RERR*DABS(ss)) RETURN
C			CONVERGENCE CRITERION MET--RETURN VALUE TO CALLING FUNCTION
			ENDIF

		s(j+1)=s(j)
		h(j+1)=0.25*h(j)
   11		CONTINUE

	    IER = 101
C		CONVERGENCE CRITERION HAS NOT BEEN MET IN MAXIMUM NUMBER OF STEPS
	RETURN

 2000		IER = 100
		s = 0.0d0
C		RERR OUT OF RANGE--RETURN VALUE OF ZERO FOR DCADRE
	RETURN
      END


	SUBROUTINE polint(xa,ya,n,x,y,dy)

	IMPLICIT NONE

      INTEGER*4	n,NMAX

      REAL*8		dy,x,y,xa(n),ya(n)

      PARAMETER	(NMAX=10)

      INTEGER*4	i,m,ns, IER

      REAL*8		den,dif,dift,ho,hp,w,c(NMAX),d(NMAX)

		IER = 0

		ns=1
		dif=DABS(x-xa(1))

		DO 11 i=1,n
			dift=DABS(x-xa(i))
			IF (dift.lt.dif) THEN
				ns=i
				dif=dift
			ENDIF
			c(i)=ya(i)
			d(i)=ya(i)
   11		CONTINUE

		y=ya(ns)
		ns=ns-1

		DO 13 m=1,n-1
			DO 12 i=1,n-m
				ho=xa(i)-x
				hp=xa(i+m)-x
				w=c(i+1)-d(i)
				den=ho-hp
				IF (den.eq.0.d0) IER = 102 
C				ERROR CODE = 102 INDICATES FAILURE IN POLINT
				den=w/den
				d(i)=hp*den
				c(i)=ho*den
   12			CONTINUE
			IF (2*ns.lt.n-m) THEN
				dy=c(ns+1)
			ELSE
				dy=d(ns)
				ns=ns-1
			ENDIF
			y=y+dy
   13		CONTINUE

      RETURN
      END



      SUBROUTINE trapzd(func,a,b,s,n)

	IMPLICIT NONE

      INTEGER*4	n

      REAL*8		a,b,s,func

      EXTERNAL	func

      INTEGER*4	it,j

      REAL*8		del,sum,tnm,x

		IF (n.eq.1) THEN
			s=0.5*(b-a)*(func(a)+func(b))
		ELSE
			it=2**(n-2)
			tnm=it
			del=(b-a)/tnm
			x=a+0.5*del
			sum=0.
			DO 11 j=1,it
				sum=sum+func(x)
				x=x+del
   11			CONTINUE
			s=0.5*(s+(b-a)*sum/tnm)
		ENDIF

      RETURN
      END



