C______________________________________________________________________________C
C                                                                              C
C    Purpose   -   This included file contains the declaration and the         C
C                  dimension statements of the program's Bessel function       C
C                  array, as well as some physical and mathematical            C
C                  constants, and the device unit numbers for the read and     C
C                  writing of output information.                              C
C                                                                              C
C    Variables -                                                               C
C         NDIMEN   This INTEGER*4 parameter specifies the maximum dimen-       C
C                  sion allowed in the Bessel function array.                  C
C                                                                              C
C         BARRAY   This is a REAL*8 array of Bessel function values of size    C
C                  NDIMEN.  The Bessel function values are determined by the   C
C                  IMSL routine MMBSJN.                                        C
C                                                                              C
C         CLIGHT   Speed of light in a vacuum, in CGS units.                   C
C                                                                              C
C         DEGRAD   Conversion constant from degrees to radians.                C
C                                                                              C
C         DFLAG    This REAL*8 parameter specifies the value of the program's  C
C                  "infinity" of flag value.  Choice of the 1.00D+36 value     C
C                  was not arbitrary, as it comes from the NCAR AUTOGRAPH      C
C                  graphics package.  This value is AUTOGRAPH's default value  C
C                  to lift the pen from the plotting surface.                  C
C                                                                              C
C         ECHARG   Magnitude of electronic charge, in CGS units.               C
C                                                                              C
C         EMASS    Mass of electron, in CGS units.                             C
C                                                                              C
C         GAMMAX   Input value of GAMMA-MAXIMUM, for the calculation of the    C
C                  the summed integrals.                                       C
C                                                                              C
C         GAMMIN   Input value of GAMMA-MINIMUM, for the calculation of the    C
C                  the summed integrals.                                       C
C                                                                              C
C         IUNITA   Unit number for  READ statements.  Typically it is 5.       C
C                                                                              C
C         IUNITB   Unit number for WRITE statements.  Typically it is 6.       C
C                                                                              C
C         MAXFCN   Approximate number of function evaluations to be permitted  C
C                  to solve via IMSL routine DMLIN (Gaussian Quadrature).      C
C                                                                              C
C         PI       Pi, to more than Real*8 precision.                          C
                                                                               
                                                                               
      INTEGER*4    IUNITA, IUNITB, MAXFCN, NDIMEN                              
                                                                               
      REAL*8       BARRAY, CLIGHT, DEGRAD, DFLAG, ECHARG, EMASS, PI
	  REAL*8       KBOLTZ                                                      
                                                                               
      PARAMETER    ( NDIMEN = 5001,                                            
     1               DFLAG  = 1.00D+36,                                        
     2               MAXFCN = 2000,                                            
     3               DEGRAD = 1.7453292519943D-02,                             
     4               PI     = 3.14159265358979323,                             
     5               ECHARG = 4.80298D-10,                                     
     6               EMASS  = 9.1091D-28,                                      
     7               CLIGHT = 2.9979256D+10,                                   
     8               IUNITA = 5,
     9               IUNITB = 6) 
	  PARAMETER    ( KBOLTZ = 1.3807D-16 )
                                                                               
      DIMENSION    BARRAY ( NDIMEN )                                           
                                                                               
  
                                                                               
C    INPUT VARIABLES
      INTEGER*4    MODE, METHOD
      REAL*8       RNU, RNUB, RNUP, P, Q, GAMMIN, GAMMAX, THETAD, 
     1             DENSTY, RERR

  
C    COMMON BLOCK TO CARRY ALL INPUT VARIABLES THROUGHOUT CODE.
      COMMON       /INPUT/ MODE, METHOD, 
     1             RNU, RNUB, RNUP, P, Q, GAMMIN, GAMMAX, THETAD, 
     2             DENSTY, RERR
  
  
C    INTERNAL WORK VARIABLES
      INTEGER*4    NS, NSMIN, NSMAX, NSCON
      REAL*8       RINDX, ATHET, STHETA, CTHETA, ERROM, THETAR
  
C    COMMON BLOCK TO CARRY ALL INTERNAL WORK VARIABLES THROUGHOUT CODE.
      COMMON       /INTRNL/ 
     1             NS, NSMIN, NSMAX, NSCON,
     2             RINDX, ATHET, STHETA, CTHETA, ERROM, THETAR
  
  
  
C    ERROR RETURN VARIABLES
      INTEGER*4    IERATH, IERBES, IERIND, IERCAT,  IERXS, 
     1             IERGAM, IEREMI,  IERNS, IERNU, IERPHI
  
C    COMMON BLOCK TO CARRY ERROR RETURN CODE VARIABLES THROUGHOUT CODE.
      COMMON       /ERBLCK/ 
     1             IERATH, IERBES, IERIND, IERCAT,  IERXS, 
     2             IERGAM, IEREMI,  IERNS, IERNU, IERPHI
                                                                               C
C______________________________________________________________________________C
