      SUBROUTINE MMBSJN(x,xnu,rj,rjp,IER)


C	LATEST REVISION	- JULY, 1999

C	PURPOSE			- TO RETURN THE VALUE OF THE BESSEL FUNCTION, rj, 
C					  AND ITS DERIVATIVE, rjp, FOR POSITIVE ARGUMENT
C					  x OF ORDER xnu.  THE RELATIVE ACCURACY IS WITHIN
C					  ONE OR TWO SIGNIFICANT DIGITS OF EPS, EXCEPT NEAR
C					  A ZERO OF ONE OF THE FUNCTIONS, WHERE EPS CONTROLS
C					  ITS ABSOLUTE ACCURACY.  FPMIN IS A NUMBER CLOSE TO
C					  THE MACHINE'S SMALLEST FLOATING-POINT NUMBER.  ALL
C					  INTERNAL ARITHMETIC IS IN DOUBLE PRECISION.  THIS 
C					  SUBROUTINE IS BASED UPON THE SUBROUTINE "BESSJY" 
C					  FOUND IN "NUMERICAL RECIPIES IN FORTRAN 77", SECOND
C			          EDITION, CAMBRIDGE UNIVERSITY PRESS, P. 236.

C	SUBROUTINES		- THESE SUBROUTINES ARE BASED UPON THOSE FOUND IN 
C					  "NUMERICAL RECIPIES IN FORTRAN 77", SECOND EDITION,
C					  CAMBRIDGE UNIVERSITY PRESS, P. 239 AND P. 187, RESP.
C		BESCHB		  CHEBYSHEV EVALUATION OF GAMMA1 AND GAMMA2
C			CHEBEV	  FUNCTION TO PERFORM EVALUATION OF CHEBYSHEV
C					  POLYNOMIAL

C	ERROR CODES		- 140 MEANS THAT X AND/OR XNU ARE OUT OF RANGE
C					- 141 MEANS THAT CONVERGENCE CRITERIA HAS NOT BEEN MET
C					  AFTER MAXIMUM NUMBER OF ITERATIONS HAVE BEEN PERFORMED
C					- 142 MEANS THAT COMPLEX CONTINUED FRACTION cf2 FAILED
C			          TO CONVERGE AFTER MAXIMUM NUMBER OF ITERATIONS
C					  HAVE BEEN PERFORMED

 	IMPLICIT NONE

      INTEGER*4	MAXIT,IER

      REAL*8		rj,rjp,x,xnu,XMIN

      REAL*8		EPS,FPMIN,PI

      PARAMETER	(EPS=1.d-16,FPMIN=1.d8*TINY(rj),MAXIT=10000,XMIN=2.d0,
     *			PI=3.141592653589793115997963468544185162d0)

      INTEGER*4	i,isign,l,nl

      REAL*8		a,b,br,bi,c,cr,ci,d,del,del1,den,di,dlr,dli,dr,e,
     *			f,fact,fact2,fact3,ff,gam,gam1,gam2,gammi,gampl,h,p,
     *			pimu,pimu2,q,r,rjl,rjl1,rjmu,rjp1,rjpl,rjtemp,ry1,
     *			rymu,rymup,sum,sum1,temp,w,x2,xi,xi2,xmu,xmu2

		IER = 0
C		ERROR CODE = 0 MEANS THAT X AND XNU ARE WITHIN RANGE AND SERIES
C		CONVERGES

		IF (x.le.0..or.xnu.lt.0.) THEN
			IER=140
C			ERROR CODE = 140 MEANS THAT X AND/OR XNU ARE LESS THAN OR EQUAL
C			TO ZERO
C			rj AND rjp ARE SET TO INFINITELY LARGE NUMBER 
			rj = HUGE(rj)
			rjp = HUGE(rjp) 
	RETURN
		ENDIF
	
		IF (x.lt.XMIN) THEN
 			nl=int(xnu+.5d0)
		ELSE
			nl=max(0,int(xnu-x+1.5d0))
		ENDIF

		xmu=xnu-nl
		xmu2=xmu*xmu
		xi=1.d0/x
		xi2=2.d0*xi
		w=xi2/PI
		isign=1
		h=xnu*xi
		IF (h.lt.FPMIN)h=FPMIN
		b=xi2*xnu
		d=0.d0
		c=h

		DO 11 i=1,MAXIT
			b=b+xi2
			d=b-d
			IF (DABS(d).lt.FPMIN)d=FPMIN
			c=b-1.d0/c
			IF(DABS(c).lt.FPMIN)c=FPMIN
			d=1.d0/d
			del=c*d
			h=del*h
			IF (d.lt.0.d0)isign=-isign
			IF (DABS(del-1.d0).lt.EPS) GOTO 1
11		CONTINUE

		IER=140
C		ERROR CODE = 140 MEANS THAT X IS GREATER THAN LARGEST POSSIBLE VALUE
C		rj AND rjp ARE SET TO INFINITELY LARGE NUMBER
		rj = HUGE(rj)
		rjp = HUGE(rjp) 
	RETURN
	
1	CONTINUE

		rjl=isign*FPMIN
		rjpl=h*rjl
		rjl1=rjl
		rjp1=rjpl
		fact=xnu*xi

		DO 12 l=nl,1,-1
			rjtemp=fact*rjl+rjpl
			fact=fact-xi
			rjpl=fact*rjtemp-rjl
			rjl=rjtemp
			IF (rjpl.gt.1.d308) GOTO 99
C	        EXIT LOOP IF rjpl ACHIEVES VALUE CLOSE TO MACHINE INFINITY
12		CONTINUE

99	CONTINUE

  		IF (rjl.eq.0.d0) rjl=EPS
		f=rjpl/rjl

		IF (x.lt.XMIN) THEN
			x2=.5d0*x
			pimu=PI*xmu

			IF (DABS(pimu).lt.EPS) THEN
				fact=1.d0
			ELSE
				fact=pimu/sin(pimu)
			ENDIF

			d=-log(x2)
			e=xmu*d

			IF (DABS(e).lt.EPS) THEN
				fact2=1.d0
			ELSE
				fact2=sinh(e)/e
			ENDIF

			call beschb(xmu,gam1,gam2,gampl,gammi)
			ff=2.d0/PI*fact*(gam1*cosh(e)+gam2*fact2*d)
			e=DEXP(e)
			p=e/(gampl*PI)
			q=1.d0/(e*PI*gammi)
			pimu2=0.5d0*pimu

			IF (DABS(pimu2).lt.EPS) THEN
				fact3=1.d0
			ELSE
				fact3=sin(pimu2)/pimu2
			ENDIF

			r=PI*pimu2*fact3*fact3
			c=1.d0
			d=-x2*x2
			sum=ff+r*q
			sum1=p

			DO 13 i=1,MAXIT
				ff=(i*ff+p+q)/(i*i-xmu2)
				c=c*d/i
				p=p/(i-xmu)
				q=q/(i+xmu)
				del=c*(ff+r*q)
				sum=sum+del
				del1=c*p-i*del
				sum1=sum1+del1
				IF (DABS(del).lt.(1.d0+DABS(sum))*EPS)GOTO 2
13			CONTINUE

			IER=135
C			ERROR CODE = 141 MEANS THAT SERIES FAILED TO CONVERGE
C			rj AND rjp ARE SET TO INFINITELY LARGE NUMBER
			rj = HUGE(rj)
			rjp = HUGE(rjp) 
	RETURN
	
2     CONTINUE

			rymu=-sum
			ry1=-sum1*xi2
			rymup=xmu*xi*rymu-ry1
			rjmu=w/(rymup-f*rymu)

		ELSE
			a=.25d0-xmu2
			p=-.5d0*xi
			q=1.d0
			br=2.d0*x
			bi=2.d0
			fact=a*xi/(p*p+q*q)
			cr=br+q*fact
			ci=bi+p*fact
			den=br*br+bi*bi
			dr=br/den
			di=-bi/den
			dlr=cr*dr-ci*di
			dli=cr*di+ci*dr
			temp=p*dlr-q*dli
			q=p*dli+q*dlr
			p=temp

			DO 14 i=2,MAXIT
				a=a+2*(i-1)
				bi=bi+2.d0
				dr=a*dr+br
				di=a*di+bi
				IF (DABS(dr)+DABS(di).lt.FPMIN)dr=FPMIN
				fact=a/(cr*cr+ci*ci)
				cr=br+cr*fact
				ci=bi-ci*fact
				IF(DABS(cr)+DABS(ci).lt.FPMIN)cr=FPMIN
				den=dr*dr+di*di
				dr=dr/den
				di=-di/den
				dlr=cr*dr-ci*di
				dli=cr*di+ci*dr
				temp=p*dlr-q*dli
				q=p*dli+q*dlr
				p=temp
				IF (DABS(dlr-1.d0)+DABS(dli).lt.EPS)GOTO 3
14			CONTINUE

			IER=136
C			ERROR CODE = 142 MEANS THAT cf2 FAILED TO CONVERGE
C			rj AND rjp ARE SET TO INFINITELY LARGE NUMBER 
			rj = HUGE(rj)
			rjp = HUGE(rjp) 

	RETURN
3     CONTINUE

			gam=(p-f)/q
			rjmu=dsqrt(w/((p-f)*gam+q))
			rjmu=sign(rjmu,rjl)
		ENDIF

		fact=rjmu/rjl
		rj=rjl1*fact
		rjp=rjp1*fact

      RETURN
      END

      SUBROUTINE	beschb(x,gam1,gam2,gampl,gammi)

	IMPLICIT NONE

      INTEGER*4	NUSE1,NUSE2

      REAL*8		gam1,gam2,gammi,gampl,x

      PARAMETER	(NUSE1=7,NUSE2=8)

      REAL*8		xx,c1(7),c2(8),chebev


      SAVE		c1,c2

      DATA		c1/-1.142022680371168d0,6.5165112670737d-3,
     *			3.087090173086d-4,-3.4706269649d-6,6.9437664d-9,
     *			3.67795d-11,-1.356d-13/

      DATA		c2/1.843740587300905d0,-7.68528408447867d-2,
     *			1.2719271366546d-3,-4.9717367042d-6,-3.31261198d-8,
     *			2.423096d-10,-1.702d-13,-1.49d-15/

		xx=8.d0*x*x-1.d0
		gam1=chebev(-1.d0,1.d0,c1,NUSE1,xx)
		gam2=chebev(-1.d0,1.d0 ,c2,NUSE2,xx)
		gampl=gam2-x*gam1
		gammi=gam2+x*gam1

      RETURN
	END


      FUNCTION	chebev(a,b,c,m,x)

	IMPLICIT NONE

      INTEGER*4	m,IER

      REAL*8		chebev,a,b,x,c(m)

      INTEGER*4	j

      REAL*8		d,dd,sv,y,y2,rj,rjp


		IF ((x-a)*(x-b).gt.0.) THEN
			IER=140
C			ERROR CODE = 140 MEANS THAT X IS OUT OF RANGE
C			rj AND rjp ARE SET TO INFINITELY LARGE NUMBER
			rj = HUGE(rj)
			rjp = HUGE(rjp) 
	RETURN
		ENDIF
      
		d=0.
		dd=0.
		y=(2.*x-a-b)/(b-a)
		y2=2.*y

		DO 11 j=m,2,-1
			sv=d
			d=y2*d-dd+c(j)
			dd=sv
11		CONTINUE

		chebev=y*d-dd+0.5*c(1)

      RETURN
      END



